unit BispectrumColorGrid;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms,
  Dialogs, ComCtrls, ToolWin, ExtCtrls, TeeProcs, TeEngine, Chart,
  SignalToolsTee, TeeEdit, SignalToolsDialogs, SpectrumAna, MtxBaseComp,
  SignalTools, SignalAnalysis, Series, StdCtrls, TeeTools, FileSignal,
  MtxDialogs, MtxVec, MtxComCtrls, TeeSurfa, MtxVecTee;

type
  TBiSpectrumGridForm = class(TForm)
    Panel1: TPanel;
    ToolBar1: TToolBar;
    ChartEditButton: TToolButton;
    ChartEditor: TChartEditor;
    RichEdit1: TRichEdit;
    SignalRead1: TSignalRead;
    BiSpectrumAnalyzer: TBiSpectrumAnalyzer;
    Panel2: TPanel;
    ProgressLabel: TLabel;
    Chart1: TChart;
    SingleLinesOnlyBox: TCheckBox;
    Series1: TMtxGridSeries;
    Button1: TToolButton;
    procedure ChartEditButtonClick(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure Button1Click(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure SingleLinesOnlyBoxClick(Sender: TObject);
  private
    { Private declarations }
  public
    { Public declarations }
    BMtx: TMtx;
  end;

var
  BiSpectrumGridForm: TBiSpectrumGridForm;

implementation

{$R *.dfm}

uses MtxVecEdit, SignalUtils, Math387;

procedure TBiSpectrumGridForm.ChartEditButtonClick(Sender: TObject);
begin
    ChartEditor.Execute;
end;

procedure TBiSpectrumGridForm.FormCreate(Sender: TObject);
begin
     SignalRead1.FileName := ExtractFileDir(Application.ExeName) + '\bz.sfs';
     BispectrumAnalyzer.Amplt.SetZero;
     BMtx := TMtx.Create;
     With RichEdit1.Lines, RichEdit1 do
     begin
        Clear;
        Add('Bicoherence shows dark blue dots where the frequency pair is related to a third frequency component ' +
            'defined as e^jw1 * e^jw2 = e^j(w1+w2). Bicoherence  ' +
            'will be 1 for a frequency pair (w1,w2) which has a product of itself (e^jw1*e^jw2) ' +
            'in the frequency spectrum. You can also compute only ' +
            'preselected frequency lines by checking the Only a few lines box.');
        Paragraph.FirstIndent := 0;
        Paragraph.Numbering := nsNone;
     end;
     Button1Click(Sender);
end;

procedure TBiSpectrumGridForm.Button1Click(Sender: TObject);
var AMtx: TMtx;
    av: TVec;
begin
    CreateIt(av);
    CreateIt(AMtx);
    try
        Screen.Cursor := crHourGlass;

        SignalRead1.RecordPosition := 0;
        SignalRead1.OverlappingPercent := 50;
        if SingleLinesOnlyBox.Checked then
        begin
            BispectrumAnalyzer.BiAnalyzer.SingleLinesOnly := True;
            BispectrumAnalyzer.BiAnalyzer.Lines.Length := 128; //compute only 256 lines
            BispectrumAnalyzer.BiAnalyzer.Lines.Ramp(64*SignalRead1.HzRes,SignalRead1.HzRes);
        end else BispectrumAnalyzer.BiAnalyzer.SingleLinesOnly := False;
        BispectrumAnalyzer.ResetAveraging;
        while BispectrumAnalyzer.Pull do
        begin
            ProgressLabel.Caption := 'Progress: ' + FormatSample('0',SignalRead1.FrameNumber/SignalRead1.MaxFrames*100) + ' [%]';
            Update;
        end;
        BispectrumAnalyzer.BiAnalyzer.Update; //compute bicoherence
        BispectrumAnalyzer.BiAnalyzer.GetFullSpectrum(BMtx,True);

        ClearFPU; //Required only for Delphi 4 and 5
        Series1.Clear;
        Series1.TopColor := clBlue;
        Series1.BottomColor := clWhite;
        Series1.Matrix := BMtx; //must be set...
        Series1.CreateDefaultPalette;
        Series1.Repaint;

        Screen.Cursor := crDefault;
    finally
        FreeIt(av);
        FreeIt(AMtx);
    end;
end;

procedure TBiSpectrumGridForm.FormDestroy(Sender: TObject);
begin
     BMtx.Destroy;
end;

procedure TBiSpectrumGridForm.SingleLinesOnlyBoxClick(Sender: TObject);
begin
     Button1Click(Sender);
end;

initialization
RegisterClass(TBiSpectrumGridForm);

end.
